'''
 Shows geographic spread of polymorphic amino acid sites in the HA segement 
 for the 2009 H1N1 'swine flu' outbreak.
 
Example:
	import H1N1
	from datetime import date
	H1N1.showSpread('HA000219', date(2009, 3, 30), date(2009, 5, 21))
'''

import GenGisAPIv1 as ggAPI
import dataHelper as dh
from datetime import date
from datetime import timedelta
import time
import colorsys

def dateStrToDate(dateStr):
	"""
	Convert date stored in a string as month, day, year (i.e. 12/1/2009) to a date object 
	"""
	from datetime import date
	
	dateSplit = dateStr.split('/')
	month = int(dateSplit[0])
	day = int(dateSplit[1])
	year = int(dateSplit[2])
	
	return date(year, month, day)
	
def lessEqualDateMDY(dateStr1, dateStr2):
	"""
	Compare two dates specified as strings in month, day, year order (i.e. 12/1/2009)
	
	@return: True if date1 <= date2, else False
	"""
	
	return dateStrToDate(dateStr1) <= dateStrToDate(dateStr2)
	
def CreateLegend(labels, colourMap):
	"""
	Create legend for figure.
	"""
	labelOffset = 25
	colour = ggAPI.Colour()
	startY = ggAPI.viewportHeight() / 2 + int(len(labels)/2)*labelOffset
	
	for i in xrange(0, len(labels)):
		colourMap.GetColour(labels[i], colour)
		marker = ggAPI.VisualMarker(colour, 8, ggAPI.MARKER_SHAPE.SQUARE, ggAPI.Point3D(40, startY-labelOffset*i, 0))
		marker.SetRenderingStyle(ggAPI.MARKER_RENDERING_STYLE.ORTHO)
		ggAPI.addMarker(marker)
		
		label = ggAPI.VisualLabel(labels[i], ggAPI.Colour(1.0, 1.0, 1.0), 18)
		label.SetScreenPosition(ggAPI.Point3D(52,startY-labelOffset*i-6,1))
		label.SetRenderingStyle(ggAPI.LABEL_RENDERING_STYLE.ORTHO)
		ggAPI.addLabel(label)
		
def showSpread(polymorphism, startDate, endDate):
	"""
	Show spread of a give polymorphism over a period of time.
	"""
	# get all location sites
	locData = dh.getLocationSetData()
	
	# set camera position
	ggAPI.setYaw(0.0)
	ggAPI.setPitch(90.0)
	ggAPI.setPosition(0.0, 0.0, 1.5785574913024902)
	ggAPI.refresh()
	
	# set initial properties of all location sites, pie charts, and sequences
	aminoAcids = set([])
	for loc in locData:
		loc.SetColour(ggAPI.Colour(1.0, 0.0, 0.0))
		loc.SetSize(2.0)
		loc.SetVisibility(False)
		
		chart = loc.GetChart()
		chart.SetField(polymorphism)
		names = chart.GetNames()
		for name in names:
			aminoAcids.add(name)
			
		chart.SetSize(10)
		chart.SetBorderSize(1)
		chart.SetAssignToOther(0)
		chart.SetChartType(ggAPI.CHART_TYPE.PIE_CHART_2D)		
		chart.SetVisibility(False)
		
		for seq in loc.sequences:
			seq.SetActive(False)
			
	aminoAcids = list(aminoAcids)
	aminoAcids.sort()
	
	# set colour map for pie charts
	colourMap = ggAPI.ColourMapDiscrete()
	for i in xrange(0, len(aminoAcids)):
		aa = aminoAcids[i]
		step = float(i) / len(aminoAcids)
		rgbColour = colorsys.hsv_to_rgb((0.2 + step) % 1.0, 0.8, 0.9)
		colourMap.SetColour(aa, ggAPI.Colour(rgbColour[0], rgbColour[1], rgbColour[2]))
		
	# assign colour map to pie charts
	for loc in locData:
		loc.GetChart().SetColourMap(colourMap)
		
	# label for showing current date
	label = ggAPI.VisualLabel("", ggAPI.Colour(0.0, 0.0, 0.0), 20)
	label.SetScreenPosition(ggAPI.Point3D(40.0, 420.0, 1.0))
	label.SetRenderingStyle(ggAPI.LABEL_RENDERING_STYLE.ORTHO)
	ggAPI.addLabel(label)
	
	# create legend
	CreateLegend(aminoAcids, colourMap)
	
	# show spread of polymorphism on a daily basis	
	dayDelta = timedelta(days=1)
	
	# get cases from start date to current simulation date
	curDate = startDate
	while(curDate <= endDate):
		startTime = time.time()
		
		for loc in locData:
			curDateStr = str(curDate.month) + '/' + str(curDate.day) + '/' + str(curDate.year)
			
			seqData = loc.sequences			
			filteredSeq = dh.filterData(seqData, 'Collection Date', curDateStr, lessEqualDateMDY)
			for seq in filteredSeq:
				seq.SetActive(True)
				
			if  len(filteredSeq) == 0:
				loc.SetVisibility(False)
				loc.GetChart().SetVisibility(False)
			else:
				loc.SetVisibility(True)
				loc.GetChart().UpdateChart()
				loc.GetChart().SetVisibility(True)
		
		# set date of label
		label.SetText(str(curDate.day) + "/" + str(curDate.month) + "/" + str(curDate.year))
				
		ggAPI.refresh()
		ggAPI.wxSafeYield()
		
		curDate = curDate + dayDelta
		
		elapsedTime = time.time() - startTime
		if elapsedTime < 1.0:
			time.sleep(1 - elapsedTime)



